<?php
	
	if(!defined('SYS_ROOT')){exit('No direct file access allowed!');}
	
	/**
	* Xtron PHP Framework
	* A MVC style framework for building web applications
	* By: Jason Alan Delaney (http://www.popcornphp.com)
	* PHP version 5+
	* 
	* Used for manipulating data in a Postgre SQL DB
	* 
	*/
	
	class postgre_dbdriver extends DB_driver{
		
		var $query = '';
		var $row;
		var $table_list = array();
		
		function __construct(){
			$this->connect_string = "host=" . $this->host . " dbname=" . $this->name . " user=" . $this->username . " password=" . $this->password;
		}
		
		public function connect(){
			return pg_connect($this->connect_string);
		}
		
		public function pconnect(){
			return pg_pconnect($this->connect_string);
		}

		public function escape_string($str){
			return pg_escape_string($str, $this->conn);
		}
		
		public function version(){
			
			$this->query("SELECT version() as `version`");
			$row = $this->row();
			return $row['version'];
			
		}
		
		/**
		* Return SQL table row
		*/
		public function row($type = 'assoc'){
			
			if($this->check_query() == false){
				return false;
			}else{
			
				if($type == 'assoc'){
					$row = pg_fetch_assoc($this->query);
				}elseif($type == 'row'){
					$row = pg_fetch_row($this->query);
				}else{
					$row = pg_fetch_array($this->query);
				}
				
				$this->row = $row;
				
				return $row;
			}
		}
		
		/**
		* Returns array from query results
		*/
		public function result(){
			
			if(count($this->result) > 0) return $this->result;
			
			$this->result = array();
			
			if($this->query != ''){
				
				while($row = $this->row('assoc')){
					$this->result[] = $row;
				}
				
				return $this->result;
				
			}
			
			return false;
			
		}
		
		/**
		* Get number of rows in a SQL table
		*/
		public function num_rows(){
			if($this->check_query() == false){
				return false;
			}else{
				return pg_num_rows($this->query);
			}
		}
		
		/**
		* Get list of tables. Return array.
		*/
		public function tables(){
			$tables = array();

			$this->query('SHOW TABLES');
			
			while($row = $this->row('row')){
				$tables[] = $row[0];
			}
			
			return $tables;
		}
		
		/*
		* Returns number of fields for given table
		*/
		function num_fields(){
			
			if($this->query == '') return 0;
			
			return @pg_num_fields($this->query);
		}
		
		
		/**
		* Get list of columns within a table
		*/
		public function columns($tbl = '*'){
			$cols = array();

			if($tbl == '*'){
				
				$tables = $this->_tables();
				
				foreach($tables as $table){
					$this->query('SHOW COLUMNS FROM ' . $table);
					while($row = $this->row('row')){
						$cols[] = $row[0];
					}
				}
				
				unset($tables);
				
			}else{
				$this->query('SHOW COLUMNS FROM `' . $this->escape_string($tbl) . '`');
				while($row = $this->row('row')){
					$cols[] = $row[0];
				}
			}
			
			return $cols;
			
		}
		
		/*
		* Selects * from a table
		*/
		function rows($table, $limit=0){
			$table = $this->escape_string($table);
			$limit = $this->escape_string($limit);
			
			$limit = ($limit <= 0)? '' : 'LIMIT ' . $limit;
			
			return $this->query("SELECT * FROM `" . $table . "` $limit", true);

		}
		
		/**
		* Backup database schema
		* This function may run slow on large databases.
		*/
		public function backup($table='all'){
			
			$bfile = SYS_ROOT . 'database' . DS . 'backup' . DS . $this->name . '_' . date("Ymd_His") . '.sql';//Backup file
			
			/*
			* Backup entire database into .sql file framework/backup
			*/
			if($table == 'all'){
				
				$tables = $this->tables();
				
				foreach($tables as $table) {
					
					
					
				}

				
				return true;
			}
			
			/*
			* Backup specific table only
			*/
			if(ctype_alpha($table)){
				
				$table = $this->escape_string($table);
				$this->query("SELECT * INTO OUTFILE '$bfile' FROM $table");
				
				return true;
			
			}
			
			return false;
			
		}
		
		/**
		* Restore a backup file
		*/
		public function restore(){
			//TODO
		}
		
		
		
		/*
		* Number of affected rows from last query
		*/
		public function affected_rows(){
			
			if($this->query == ""){
				return 0;
			}
			
			return (int)pg_affected_rows($this->query);
		}
		
		
		/*
		* Check database connection
		*/
		public function ping(){
			
			if($this->conn == false) return false;
			
			return @pg_ping($this->conn);
		}

		/*
		* Free memory allocated from the result command
		*/
		function free_result(){
			
			if($this->check_query()){	
				return @pg_free_result($this->query);
			}
			
			return false;
			
		}
		

		/**
		* Run SQL query
		*/
		public function query($sql, $return = false){
			
			if($sql == ''){
				if($this->debug){
					error("Invalid SQL query");
				}
				return false;
			}
			
			$this->query = pg_query($sql, $this->conn) or die(error(pg_last_error($this->conn), 'SQL Error'));
			$this->result = array();//clear result cache
			
			if($return){
				return $this->query;
			}
		}
		
		/*
		* Fixes the index id columns of a table to be 1-x
		*
		* Example: you have a table with 5 rows with auto incrimenting`id` columns 
		* and they are listed as "1,4,5,9,2". This function will arrange them to read "1,2,3,4,5"
		*
		* Especially good for OCD people...
		*/
		public function organize_index($table){
			if($table == null || $table == '') error("Undefined table given. DB::organize_index()", "SQL Error: ");
			
			$table = $this->escape_string($table);
			
			$this->query("SELECT `id` FROM `" . $table . "` ORDER BY `id` ASC");
			
			$current = 1;
			foreach($this->result() as $row){
				
				$this->query("UPDATE `" . $table . "` SET `id`='" . $current . "' WHERE `id`='" . $row['id'] . "'");
				$current += 1;
				
			}
			
			return true;
			
		}
		
		/**
		* Close DB connection
		*/
		public function close(){
			pg_close($this->conn);
			$this->conn = false;
		}
		
	}
	
?>