<?php
	
	if(!defined('SYS_ROOT')){exit('No direct file access allowed!');}
	
	/**
	* Xtron PHP Framework
	* A MVC style framework for building web applications
	* By: Jason Alan Delaney (http://www.popcornphp.com)
	* PHP version 5+
	* 
	*  A basic driver to talk to a redis instance
	*/
	
	// \r\n
	define('CRLF', sprintf('%c%c', 13, 10));
	
	class redis_dbdriver extends DB_driver{
		
		private $_socket 	= '';
		private $_connected = false;
		
		function __construct($params){
			
			$this->config = load_class('Config');
			
			parent::__construct($params);
			$this->socket_connect();
			
		}
		
		/*
		* DB driver persistant connect (not used here)
		*/
		public function pconnect(){
			$this->socket_connect();
		}
		
		/*
		* Generic wrapper for DB driver connect func.
		*/ 
		public function connect(){
			$this->socket_connect();
		}
		
		/*
		* Open a connection to the host through TCP
		*/
		public function socket_connect(){
			$addr = 'tcp://' . $this->host;
			
			$port = (int) $this->port;
			
			$this->_socket = @fsockopen($addr, $port, $errno, $errstr);
			
			if(!$this->_socket || !is_resource($this->_socket)){
				error('Unable to connect to TCP socket on host: ' . $this->host . ' Error: ' . $errstr, 'Socket Error');
			}
			
			$this->_connected = true;
			
			return $this->_socket;
		}
		
		/*
		* Connects to a TCP host using stream_socket_connect
		*/
		public function socket_stream_connect(){
			
			$addr = 'tcp://' . $this->host . ':' . $this->port;
			
			$flags = STREAM_CLIENT_CONNECT;
			
			/*
			* Persistent connection?
			*/
			if($this->pconnect){
				$flags = STREAM_CLIENT_PERSISTENT;
			}
			
			$socket = @stream_socket_connect($addr, $errno, $errstr, $flags);
			
			$this->_connected = true;
			
			return $socket;
		}
		
		public function close(){
			$this->socket_disconnect();
		}
		

		public function socket_disconnect(){
			if($this->_connected){
				fclose($this->_socket);
			}
			
			$this->_socket = '';
			$this->_connected = false;
			
		}
		
		/*
		* Returns socket connection
		*/
		private function socket(){
			if($this->_socket == ''){
				$this->socket_connect();
			}
			
			return $this->_socket;
		}
		
		/*
		* Writes data to socket / sends command to Redis
		*
		* See http://redis.io/topics/protocol for the Redis protocol spec.
		*
		* This code was ported to PopcornPHP from phpkode.com
		* Original by Frederic G. stby
		*/
		public function socket_write($name, $args) {
			$socket = $this->socket();

			array_unshift($args, strtoupper($name));

			$command = '*' . count($args) . CRLF;

			foreach($args as $arg){
				$command .= '$' . strlen($arg) . CRLF . $arg . CRLF;
			}

			// Send command to server
			fwrite($socket, $command);

			$response = trim(fgets($socket, 512));

			// Handle response
			switch(substr($response, 0, 1)){
			
				case '-':// error message
					
					error(substr($response, 5), 'Socket Error');
				
				break;
				case '+': //single line reply
				case ':': //integer number reply
				
					return substr($response, 1);
					
				break;
				case '$': //bulk reply
					
					if($response === '$-1'){ 

						return null; 
					
					}else{
						
						$length = substr($response, 1) + 1;
						return fgets($socket, $length);
						
					}
					
				break;
				case '*': // multi-bulk reply
					if($response === '*-1' || $response === '*0'){ return null; }

					$data = array();

					$count = substr($response, 1);

					for($i = 0; $i < $count; $i++){
					
						$length = substr(trim(fgets($socket)), 1) + 1;
						$data[] = trim(fgets($socket, $length + strlen(CRLF)));
					
					}
					
					return $data;
					
			}
			
		}
		
		
		/*
		* Get the value of a given key
		*/ 
		function get($key){
			if($key == '' || is_int($key)){
				echo 'invalid key given';
				return false;
			}
			
			$value = $this->socket_write('get', array($key));
			
			if($this->config->item('display_errors') == true){
				
				if($value == false || $value == null){
					
					echo 'No response given by server.';
					
					return false;
					
				}
				
			}
			
			return $value;
			
		}
		
		/*
		* Set a value to a new key
		*/
		function set($key, $val){
		
			return $this->socket_write('set', array($key, $val));
			
		}
		
		/*
		* Handle commands that aren't wrapped
		*/
		function __call($name, $args){
			
			if(!is_array($args)){
				
				$args = array($args);
			
			}
			
			return $this->socket_write($name, $args);
			
		}
		
		/*
		* Disconnect on close
		*/
		public function __destruct(){
			
			if(is_resource($this->_socket)){
				
				fclose($this->_socket);
				
			}
			
		}
		
	}
	
?>