<?php
	
	if(!defined('SYS_ROOT')){exit('No direct file access allowed!');}
	
	/**
	* Access Control List Library
	*/
	
	/**
	* Login/out & protected content / auth requirement features.
	*/
	
	class Acl {
		
		public $user;//User name
		private $xpf;
		private $expire;//amount of seconds for a cookie to expire
		
		function __construct(){
			$this->xpf =& controller_instance();
			$this->xpf->load->model('acl_model','',true);
		}	
		
		//Allow access to page to specific user level
		//Default values: 0 = Admin; 1 = Member; 2 = Guest
		public function access($level=1){
			
			//Text based level given?
			if(!is_numeric($level)){
				switch($level){
					case 'admin':
						$level = 0;
						break;
					case 'user':
						$level = 1;
						break;
					case 'guest':
						$level = 2;
						break;
				}
			}
			
			//Guests?
			if($level >= 2){
				return;
			}
			
			//Get user level if logged in.
			$l = ($this->is_logged())? $this->level() : 2;

			//If user is NOT properly logged in or is not at low enough level.
			if(!$this->is_logged() || $l > $level){
				//Deny access to page if user is not at high enough level
				error('You are not authorized to view this page.','Authorization Error:');
			}
			return;
		}
		
		/**
		* TODO:
		* - Use function to set class flag to one for logged in, when logged out set to 0.
		* This will eliminate having to call this function so many times.
		*/
		
		//Check if user is logged in, return bool
		public function is_logged($chksess=true){
		
			//Using sessions, but is the USER_ID var set?
			if($chksess == true && $this->xpf->config->item('session_auto_start') == true){
				if(!isset($_SESSION['USER_ID'])){
					return false;
				}
			}
		
			if(isset($_COOKIE['idrt']) && isset($_COOKIE['xpfcode']) ){
				$code = $this->str_rot_encode($this->user_from_rot($_COOKIE['idrt']) . 'edocfxp'); 
				if($_COOKIE['xpfcode'] == $code){
					return true;
				}
				return false;
			}
			return false;
		}
		
		/**
		* TODO:
		* - View file support
		*/
		//Allow access to certain content on a page
		public function access_content($content, $level=2){
			//return the HTML if user is at high enough level to access
			if($this->is_logged() && $this->level() <= $level){
				return htmlentities($content);
			}
			return null;
		}
		
		public function level(){
			if($this->is_logged()){
				//Get logged user
				$user = $this->user_from_rot($_COOKIE['idrt']);
				return (int)$this->xpf->acl_model->uinfo($user,'level');
			}else{
				return 2;//Guest level
			}
		}
		
		//Check if a user is an administrator
		public function is_admin(){
			$level = $this->level();
			
			if($level == 0) return true;
			
			return false;//not admin
			
		}
		
		//Decode the secret string
		private function str_rot_decode($str){
			return base64_decode(str_rot13($str));
		}
		
		//Encode a string into a secret
		private function str_rot_encode($str){
			return str_rot13(base64_encode($str));
		}
		
		//Get user name from rotten string
		private function user_from_rot($str){
			if($str == ''){return;}
			
			$user = $this->str_rot_decode($str);
			$user = str_replace('/s','',$user);
			return $user;
			
		}
		
		/**
		* Generate hashed password
		*/
		public function generate_password($pass){
			return hash('sha256', $pass);
		}
		
		//Log user in (expires in 1 hour by default)
		//@string pass = users password (real text, not the hash)
		//@string user = the user name
		//@int exp = the time to expire a users login session (in seconds).
		public function login($user, $pass, $exp=3600){
			if(empty($user) || $user == ''){
				return;
			}
			
			//User logged in already? idrt = user
			if(isset($_COOKIE['idrt']) || isset($_COOKIE['xpfcode'])){
				
				/*
				* If sessions are enabled, and the user has cookies set but no 
				* session variable, then log them out first.
				*/
				if($this->xpf->config->item('session_auto_start') == true){
					
					$this->logout();
					
				}else{
				
					return;
					
				}
			}
			
			$this->expire = (int)$exp;
			
			//Escape changes of SQL injection
			$pass = $this->xpf->db->escape_string($pass);
			$user = $this->xpf->db->escape_string($user);
			
			$pass = strip_tags($pass);
			$user = strip_tags($user);
			
			//Get password from DB for the user
			$storpass = $this->xpf->acl_model->uinfo($user, 'password');
			
			if($storpass == hash('sha256', $pass)){
				$time 		= time()+$exp;
				$rstring 	= $this->str_rot_encode($user . 'edocfxp'); 
				$encuser	= $this->str_rot_encode('/s'.$user.'/s');//Use user_from_rot() to decode this
				$this->user = $user;//Username
				$folder		= ($this->xpf->config->item('folder') != '')? '/'.$this->xpf->config->item('folder') : '/';
				
				
				/*
				* User name cookie
				*/
				setcookie("idrt", $encuser, $time, $folder);
				
				/*
				* Encoded string used for authentication
				*/
				setcookie("xpfcode", $rstring, $time, $folder);
				
				//If we have sessions enabled then we can save the user ID in the session array
				if($this->xpf->config->item('session_auto_start') == true){
					
					$_SESSION['USER_ID'] 	= $this->xpf->acl_model->uinfo($user, 'ID');
				
				}
				
				//Get rid of sensitive information
				unset($pass);
				unset($user);
				return true;
				
			}else{
			
				return false;
				
			}
			
			
		}

		
		//Log user out
		public function logout(){
			//User is logged in?
			if($this->is_logged()){
				$time = time()-$this->expire;
				
				//cookie domain
				$folder		= ($this->xpf->config->item('folder') != '')? '/' . $this->xpf->config->item('folder') : '/';
				
				setcookie('idrt', '', $time, $folder);//User name
				setcookie('xpfcode', '', $time, $folder);//auth cookie
				
				unset($_COOKIE);
				
				if($this->xpf->config->item('session_auto_start') == true){
					
					unset($_SESSION['USER_ID']);
				
				}
				
			}
			return;
		}
		
		
	}
	
?>