<?php
	
	if(!defined('SYS_ROOT')){exit('No direct file access allowed!');}
	
	/**
	* Xtron PHP Framework
	* A MVC style framework for building web applications
	* By: Jason Alan Delaney (http://www.popcornphp.com)
	* PHP version 5+
	*
	* Manipulate cookie headers
	* This class uses 0.024Mb of ram when called
	*/
	
	class Cookie{
		
		function __construct(){
			//Load the app config items
			$this->config =& load_class('Config');
		}
		
		/*
		* Sets new cookie. Returns true on success
		* @array the cookie data (key, val, etc..)
		* @bool use cookie only in secure connections
		*/
		function new_cookie($name, $val, $data=array(), $https=false){
			//Cookies are disables
			if($this->config->item('enable_cookies') !== true){
				return false;
			}
			
			if(is_array($data)){
				
				if($name == ''){
					error("You must specify a cookie name. Cookie::new_cookie()", "Cookie Error: ");
					return false;
				}
				
				if($val == ''){
					error("You must specify a cookie value. Cookie::new_cookie()", "Cookie Error: ");
					return false;
				}
				
				//Time for cookie to expire (default is, until browser is closed)
				$exp = (isset($data['expire']))? time() + (int) $data['expire'] : 0;
				
				//Path where cookie is available (default is /)
				$path = (isset($data['path']))? (string) $data['path'] : '/';
				
				//The domain name of the cookie (default)
				$domain = (isset($data['domain']))? (string) $data['domain'] : '';
				
				return setcookie($name, $val, $exp, $path, $domain, $https);
				
			}
			return false;
		}
		
		/*
		* Remove a cookie. Returns bool true on success
		* @string the cookie key
		*/
		function delete_cookie($key){
			if($key == '') return false;
			
			if(isset($_COOKIE[$key])){
				
				setcookie($key, '', 1);
				unset($_COOKIE[$key]);
				return true;
			}
			
			return false;
			
		}
		
		/*
		* Returns a cookie value
		* @string the cookie key
		* @bool suppress error on non-existant cookie
		*/
		function get_cookie($key, $suppress=false){
			
			if(isset($_COOKIE[$key])){
				
				return $_COOKIE[$key];
				
			}elseif($suppress === false){
				
				minor_error('Cookie "' . $key . '" not found.');
				
			}
			
			return false;
			
		}
		
	}
	
?>