<?php	
	
	if(!defined('SYS_ROOT')){exit('No direct file access allowed!');}
	
	/**
	* Xtron PHP Framework
	* A MVC style framework for building web applications
	* By: Jason Alan Delaney (http://www.xtroncms.com)
	* PHP version 5+
	*/	
	
	class Display {
		
		var $output;
		var $http;
		var $config;
		var $path;
		var $template;//View file name
		
		function __construct(){
			$this->config = load_class('Config');
		}
		
		/**
		* Check if a cached version of view file is avalable
		* If cached file is more than a day old, return false.
		*/
		function cache_exists($name){
			//cached file location
			$cname 	= SYS_ROOT . 'cache' . DS . 'cache.' . str_replace('.php','',basename($name)) . '.php';
			if(is_file($cname)){
				return true;
			}else{
				return false;
			}
		}
		
		/*
		* Check if view file exists in the default directory
		*/
		function exists($name){
			
			$name = basename($name);
			
			/*
			* View path (without file extension)
			*/
			$dir = APP_ROOT . 'views' . DS . $name;
			echo $dir;
			/*
			* tpl of PHP file exist?
			*/
			if(file_exists($dir . '.tpl') || file_exists($dir . '.php')){
				return true;
			}
			
			return false;
			
		}
		
		/**
		* Show template file with caching features
		* @name template name (no file extension)
		*/
		function view($name, $data = null, $cache = false) {

			/**
			* TODO: 
			* - Controller _output override 
			*/
			
			$this->path = (is_file($name))? $name : APP_ROOT . 'views' . DS . basename($name) . '.php';
	
			//Maybe a sub-category given?
			if(!is_file($this->path)){
				$this->path = APP_ROOT . 'views' . DS . $name . '.php';
			}
			
			if(!is_file($this->path) && $cache == false){
				error('Template not found: ' . $this->path, 'Display Error: ');
			}
			
			//Cached file location
			$cname = SYS_ROOT . 'cache' . DS . 'cache.' . basename($name,'.php') . '.php';
			
			if($cache == true && is_file($cname)){
				
				/*
				* Browser cache
				*/
				$cache_file = file_get_contents($cname);
				$id			= md5($cache_file);
				$lastchange = filemtime($cname);
				
				$headers = apache_request_headers();
				
				//Browser cache expiration
				header('Cache-Control: max-age=' . 3600);
				header('Expires: '.gmdate('D, d M Y H:i:s', time()+3600).' GMT');
				header('Last-Modified: '.gmdate('D, d M Y H:i:s', $lastchange).' GMT');
				//Tells te browser if the content has been changed
				header('ETag: ' . $id);

				$update = !(isset($headers['If-Modified-Since']) && strtotime($headers['If-Modified-Since']) == $lastchange);
				$match  = (isset($headers['If-None-Match']) && ereg($id, $headers['If-None-Match']));
				
				//If the page has not been modified or content is the same
				if(!$update || $match){
					//All we need to do is tell the browser that there is no new content to download
					header('HTTP/1.1 304 Not Modified');
					header('Connection: close');
				}else{
					header("HTTP/1.1 200 OK");
					return $cache_file;
				}
				
			}else{
				//Pass variables to view page
				if($data != null && is_array($data)){
					foreach($data as $vname => $val){
						//Set variable name from the array value variable. "variable reflection"
						$$vname = $val;
						unset($val);
					}
				}
				
				/**
				* Read template in output buffer, save cache version if applicable,
				* then save contents to output var.
				*/
				if($this->config->item('enable_compression') === true){
					if(isset($_SERVER['HTTP_ACCEPT_ENCODING']) && strpos($_SERVER['HTTP_ACCEPT_ENCODING'], 'gzip') != FALSE){
						ob_start('ob_gzhandler');
					}else{
						//Configuration is set to use compression but it's not available =[
						ob_start();
					}
				}else{
					//Configuration is set to not use compression
					ob_start();
				}

				include($this->path);
				$buffer = ob_get_contents();
				
				ob_end_clean();
				
				//Save cached version
				if($cache == true && !is_file($cname)){
					//Save the cached file 
					if(substr(basename($this->template),0, 1) !== '_'){
						//open/create cache file for writing
						$fh		= fopen($cname,'w+'); 
						fwrite($fh, $buffer); 
						fclose($fh);
					}
				}

				return $buffer;
			}
		}

	}
	
?>