<?php
	
	if(!defined('SYS_ROOT')){exit('No direct file access allowed!');}
	
	/**
	* PopcornPHP
	* A MVC style framework for building web applications
	* By: Jason Delaney (http://www.popcornphp.com)
	* PHP version 5+
	*
	* File loader class
	*/
	
	class Fetcher {
		
		var $models = array();//Loaded models
		var $addons = array();//Loaded addons
		var $classes = array();//Loaded libraris
		
		/**
		* Load library file
		* @string name = name of the class (if different from the file name)
		*/
		public function lib($lib, $params = null, $name = null){
			
			if($params != null){
				if(!is_array($params)){
					$params = array($params);
				}
			}
			
			//Load an array of Libraries :]
			if(is_array($lib)){
				foreach($lib as $class){
					$this->_fetch_class(ucfirst($class), null, $name);
				}
			}else{
				$this->_fetch_class($lib, $params, $name);
			}
			
		}
		
		/**
		* Load addons
		*/
		public function addon($addons = array()){
			
			$addons = (is_array($addons))? $addons : array($addons);
			
			foreach($addons as $addon){
				
				$addon_base_name = str_replace('.addon','',str_replace('.php','',$addon));
				
				$sysaddon = SYS_ROOT . 'addons' . DS . $addon_base_name . '.addon.php';
				$appaddon = APP_ROOT . 'addons' . DS . $addon_base_name . '.addon.php';
				
				//Dont load addon twice
				if(@$this->addons[$addon_base_name]){
					return false;
				}
				
				if(file_exists($sysaddon)){
				
					include_once($sysaddon);
					
				}elseif(file_exists($appaddon)){
					
					include_once($appaddon);
					
				}else{
					//Someone's playing tricks...
					error('System addon not found: ' . $addon);
					return;
					
				}
				
				//Tell ourselves we've loaded this addon
				$this->addons[$addon_base_name] = true;
				
			}
			
		}
		
		/**
		* Load a model class
		* @model = Model name
		* @name = Class name
		* @db = Database connection
		*/
		public function model($model, $name = '', $db = true){
			/**
			* TODO: Handle models in sub-folders
			*/
			
			if($model == '' || empty($model)){
				return;
			}
			
			//More than one model?
			if(is_array($model)){
				if(!is_array($name) && $name != ''){
					error('You must specify an array of model names or leave the second parameter empty to use the model names as the class names.');
					return;
				}
				
				$i=0;
				$flag = (is_array($name))? true : false;
				foreach($model as $mname){
					$db = (strpos($mname,':db'))? true : false;
					//Take the db connect flag out of the model name.
					$mname = str_replace(':db','',$mname);
					$cname = ($flag)? $name[$i] : $mname;
					//Run this function for the model
					$this->model($mname, $cname, $db);
					$i++;
					
				}
				return;
			}
			
			
			//No class name defined, so assume it's the same as the model file name
			if($name == ''){
				$name = $model;
			}
			
			
			//Already loaded this model?
			if(in_array($model, $this->models)){
				return;
			}
			
			//Get reference of main controller
			$xpf =& controller_instance();
			
			//Check if the 
			if(isset($xpf->$name)){
				error('The model name "' . $name . '" is already being used as another resource.');
				return;//In case display_errors is disabled we still want to exit the function
			}

			$model = strtolower($model);
			
			//Make sure the dang file even exists
			if(file_exists(APP_ROOT . 'models' . DS . $model . '.php')){
				
				//Load the base model class if that hasn't been done already
				if(!class_exists('Model')){
				
					load_class('Model');
					
				}
				
				//Load the database functions too?
				if($db == true && !class_exists('DB_driver')){
					
					$xpf->load->database();
					
				}
				
				require_once(APP_ROOT . 'models' . DS . $model . '.php');
				
				$model = ucfirst($name);
				
				$xpf->$name = new $model();

				/**
				* Run the base model function _get_libs() to get all references of loaded libraries
				*/
				$xpf->$name->_get_libs();

				/**
				* Assign model name to models array so we know that it's already loaded
				*/
				$this->models[] = $name;
				
			}else{
				error('Model file not found in ' . APP_FOLDER . DS . 'models' . DS . $model);
				return;
			}
			
		}
		
		/**
		* Load database driver files and start DB connection
		*/
		public function database($settings = array()){
			
			$xpf = controller_instance();
			
			//If the database object is already loaded, don't load it again
			if(isset($xpf->db) && is_object($xpf->db)){
				return false;
			}
			
			require_once(SYS_ROOT . 'database' . DS . 'database.php');
			
			
			$xpf->db =& db($settings);
		}
		
		/**
		* Loads requested class
		* We can't just use "load_class" because this function is for humans
		* and we need more functionality and error handling for that.
		*/
		private function _fetch_class($class, $params = null, $name = null){

			$params = ($params != null)? $params : '';
			
			if($class == '' || empty($class) || $class == null){
				return;
			}
			
			//Remove ".php" in case it was supplied, we don't want it
			//Also trim trailing slashes
			$class = str_replace('.php','',trim(trim($class,'/'),'\\'));

			//In case there is no path
			$path = '';
			
			//Class in a different directory?
			if(strpos('/', $class)){
				
				//Put path into different parts
				$path = explode($class);
				
				//Class ended up at the end of that array
				$class = end($path);
				
				//Put the path back into a string
				$path = implode($path , '/');
				
			}

			if(file_exists(APP_ROOT . 'libs' . DS . $path . $class . '.php')){
			
				include(APP_ROOT . 'libs' . DS . $path . $class . '.php');
				$this->_start_class($class, $params, $name);
				
			}elseif(file_exists(SYS_ROOT . 'libs' . DS . $path . $class . '.php')){
				
				include(SYS_ROOT . 'libs' . DS . $path . $class . '.php');
				$this->_start_class($class, $params, $name);
				
			}else{
				error('Unable to fetch class \'' . $class . '\' in ' . __FILE__ . ' check autoload.php');
				
			}
			
		}
		
		/**
		* Starts a specified class
		* Sets the class object into a variable in the main controller
		*/
		private function _start_class($class, $params = null, $cname = null){
			
			//If the class doesn't really exist
			if(!class_exists($class) && $name === null){
				
				error('Class ' . $class . ' does not exist!');
				return false;
				
			}else{
				
				/*
				* Has class already been defined?
				*/
				if(isset($this->classes[strtolower($class)])){
					return false;
				}
				
				//Get instance of Controller class
				$instance =& controller_instance();
				
				$name = strtolower($class);
				
				$class = ($cname === null)? $class : $cname;
				
				$params = ($params != null)? implode(',', $params) : null;
				
				//Set main controller class variable to loaded class object
				$instance->$name = new $class($params);
				
				//add to defined classes list
				$this->classes[strtolower($class)] = $class;
				
				return true;
			}
			
		}
		
		/*
		* Loads objects given from the autoload config file
		*/
		function xpf_autoloader(){
			include(APP_ROOT . 'config' . DS . 'autoload.php');
			
			if(!isset($autoload)){
				return false;
			}
			
			if(isset($autoload['addons'])){
				$this->addon($autoload['addons']);
			}
			
			if(isset($autoload['library'])){
				$this->lib($autoload['library']);
			}
			
			if(isset($autoload['models'])){
				
				$datab = false;
				
				foreach($autoload['models'] as $model){
					if(strpos($model, ':db') !== false){
						$model = substr($model,0,-3);
						$datab = true;
					}
					
					$this->model($model, '' , $datab);
				}
				
			}
		}
		
	}
	
?>