<?php
	
	if(!defined('SYS_ROOT')){exit('No direct file access allowed!');}
	
	/**
	* Xtron PHP Framework
	* A MVC style framework for building web applications
	* By: Jason Alan Delaney (http://www.xtroncms.com)
	* PHP version 5+
	*
	*/
	
	class Mod {
		
		var $config;
		var $enable;
		var $points = array();
		var $loaded;//Loaded mods
		
		function __construct(){
			$this->config =& load_class('Config');
			
			//Valid mod invoke points
			$this->points = array(
				'pre_controller',
				'post_controller',
				'pre_core',
				'post_core'
			);
			
			if($this->config->item('enable_mods') == false){
				return;
			}
			
			//Mods config file (hope it exists)
			include_once(APP_ROOT . 'config' . DS . 'mods.php');
			
			if(!is_array($mods)){
				error('Invalid Mod config file', 'Mod Error');
			}
			
			$this->mods =& $mods;
		}
		
		/**
		* Load all mods for a certain point
		*/
		function at_point($point = ''){
			/**
			* Are mods disabled?
			*/
			if($this->config->item('enable_mods') == false){
				return false;
			}
			
			//Mod point defined?
			if($point == '' || !in_array($point, $this->points)){
				minor_error('Invalid mod point "' . $point . '" supplied.','Mod Error');
				return false;
			}else{
			
				if(isset($this->mods[$point]) && !empty($this->mods[$point])){
					if(is_array($this->mods[$point])){
						foreach($this->mods[$point] as $mod){
							$this->start_mod($mod);
						}
					}else{
						$this->start_mod($this->mods[$point]);
					}
				}
				
			}
			
			return true;
			
		}
		
		/**
		* Calls a given mod
		*/
		function start_mod($mod){
			
			/**
			* TODO: Use XML library for reading config file
			*/
			
			if(is_array($mod)){
				return false;
			}

			//Remove trailing '/'
			$dir = (substr($mod, -1) == '/')? substr($mod, 0, strlen($mod)-1) : $mod;
			
			$base_mod_name = basename($mod,'.mod.php');
			
			$mod = (strpos($mod, '.mod'))? $mod : $dir . '.mod.php';

			$loc = SYS_ROOT . 'mods' . DS . $mod;
			
			if(file_exists($loc)){
				
				//Get config file for mod
				if(!file_exists(SYS_ROOT . 'mods' . DS . $dir . '.xml')){
					
					$this->generate_config($dir);
					
				}
				
				//Get config file
				$xml = simplexml_load_file(SYS_ROOT . 'mods' . DS . $dir . '.xml');
				
				//Get class name (if there is one) and make sure it's just a string
				$class 		= (isset($xml->class) && $xml->class != '')? (string)$xml->class : false; 
				$function 	= (isset($xml->function) && $xml->function != '')? (string)$xml->function : false; 
				
				$params		= $xml->params;
				
				if($class !== false){

					if(!class_exists($class)){
						require($loc);
					}
					
					if(!class_exists($class)){
						error('Class "' . $class . '" not found.', 'Mod Error');
					}
					
					$modClass = new $class();
					$modClass->$function($params);
					
					$this->loaded[] = $base_mod_name;
					
				}elseif($function !== false){
					
					if(!function_exists($function)){
						require($loc);
					}
					
					$function($params);
					
					$this->loaded[] = $base_mod_name;
					
				}else{
					//Don't know what to load!
					return false;
					
				}
			
			}else{
				//No mod found
				error('Mod file not found: "' . $loc . '"','Mod Error');
			}
			
		}
		
		/**
		* Creates generic xml config file for a mod
		*/
		function generate_config($filename, $function = '', $class = '', $params = ''){
			/**
			* TODO: Make use of the XML library
			*/
			
			//Make sure params is an array
			$params = (!is_array($params))? array(null) : $params;
			
			$file = SYS_ROOT . 'mods' . DS .$filename . '.xml';
			
			$xmldata = '<?xml version="1.0" encoding="ISO-8859-1"?> ' .  PHP_EOL . '<mod>' .  PHP_EOL;
			
			$xmldata .= '<class>' . $class . '</class>' .  PHP_EOL;
			$xmldata .= '<function>' . $function . '</function>' . PHP_EOL;
			
			foreach($params as $param){
				$xmldata .= "<param> " . $param . "</param>" . PHP_EOL;
			}
			
			$xmldata .= '</mod>';
			
			//Write the actual file
			$fh = fopen($file, 'w+');
			fwrite($fh, $xmldata);
			fclose($fh);
			
		}
		
		/**
		* Returns array of loaded mods
		*/
		function get_loaded_mods(){
			
		}
		
	}
	
?>