<?php	
	
	if(!defined('SYS_ROOT')){exit('No direct file access allowed!');}
	
	/**
	* Xtron PHP Framework
	* A MVC style framework for building web applications
	* By: Jason Alan Delaney (http://www.xtroncms.com)
	* PHP version 5+ 
	*/	
	
	/**
	* Determines what controller and method to load from URI
	*/
	
	class Router {
		
		var $controller;//controller to be routed to
		var $default_controller;//app default controller
		var $directory;//controller directory
		var $uri;//used to hold URI object
		var $routes;//used for retriving $route array from app/config/routes.php
		var $method = 'index';//controller method
		var $req_method;//routed method
		var $req_controller;//routed controller
		var $protected_methods = array();//Controller method name to protect against users from accessing via URI
		var $root;//Holds root directory of controller
		
		function __construct(){
			
			$this->uri =& load_class('URI');
			$this->config =& load_class('Config');
			$this->_set_route();
		}
		
		function _set_route(){
			
			/**
			* Standard query support
			*/
			if($this->config->item('clean_uri') === false){
				//Get controller for URI 
				$this->controller = trim($this->uri->_clean($_GET[$this->config->item('controller_trigger')]));
				
				//Was a method defined too?
				if(isset($_GET[$this->config->item('method_trigger')])){
					$this->method = trim($this->uri->_clean($_GET[$this->config->item('method_trigger')]));
				}
				return;
			}
			
			//get app routes config file
			require_once(APP_ROOT . 'config' . DS . 'routes.php');
			$this->routes = (!isset($route) || !is_array($route))? array() : $route; 
			unset($route);
			
			$this->root = APP_ROOT;
			
			/**
			* Check for methods to protect from routes config file
			*/
			$this->_protect_methods();
			
			//set default controller. If not set in config file then use "intro" as default
			$this->default_controller = ($this->routes['default_controller'] == '' || !isset($this->routes['default_controller']))? false : strtolower($this->routes['default_controller']);
			
			//Get route (URI), set to URI::uri_string
			$this->uri->_get_uri();
			
			$this->uri->uri_string = strip_tags($this->uri->uri_string);
			
			//If String contains nothing then use defaults
			if($this->uri->uri_string == '' || $this->uri->uri_string == null){
				
				//No default controller? Oh no!
				if($this->default_controller == false){
					error('No default controller is defined in your routes config file');
				}
				
				//Set controller to default
				if(strpos($this->default_controller, '/') == false){

					$this->set_req(array($this->default_controller,'index'));
					
				}else{
					//Set controller to one defined in URI
					$uri = explode('/',$this->default_controller);		
					
					$this->set_req($uri);
				}
				
				return;
				
			}else{
				//Put URI into segments
				$this->uri->get_segments();

				//Route system 
				$this->set_req($this->uri->uri_segments);
				
				unset($this->uri->segments[0]);
				unset($this->uri->uri_resegments[0]);
			}
		}
		
		/**
		* Set URI request from segments. Handle controller and method data
		*/ 
		function set_req($segments){
			$this->controller = $segments[0];
			$this->method = (isset($segments[1]))? $segments[1] : 'index';
			//Look for controller in framework dir.
			if(file_exists(SYS_ROOT . 'controllers' . DS . $segments[0] . '.php')){
				$this->root = SYS_ROOT;
				$this->uri->uri_resegments = $segments;
				return;
			}
			// Does the requested controller exist in the root folder?
			if (!file_exists(APP_ROOT . 'controllers' . DS . $segments[0] . '.php')){
				if(is_dir(APP_ROOT . 'controllers' . DS . $segments[0])){
					$this->directory = ($segments[0]);
					$segments = array_slice($segments, 1);
					
					if(count($segments) > 0){
						
						//Controller does not exist, show 404.
						if(!file_exists(APP_ROOT . 'controllers' . DS . $this->directory . $segments[0] . '.php')){
							error('404 Not Found','Page Not Found');
						}
						
					}else{
						//No URI segments, set controller to default.
						$this->method 		= 'index';
						$this->controller 	= $this->default_controller;
						echo 'i';
						if(!file_exists(APP_ROOT . 'controllers' . DS . $this->directory . $this->default_controller . '.php')){
							$this->directory = '';
						}
					
					}
					
				}else{
					//Invalid controller
					error('','','404');
				}
			}else{
				//Controller from URI exists, use it.
				if (count($segments) == 0){
					return;
				}
				//Controller and method is already set to default at the beginning of this function.
			}

			//Make sure requested method isn't "protected"
			if(in_array($this->method, $this->protected_methods)){
				$this->controller 	= $this->default_controller;
				$this->method		= 'index';
			}
			$this->uri->uri_resegments = $segments;
		}
		
		//Get controller and function from URI object
		function _get_controller(){
			
			$parts = explode('/', $this->uri->uri_string);

			//If the controller segment of $parts is empty and it's not the only segment in $parts, slice it out.
			//This is to fix some issue with index.php being part of the URI and it being caught in the $parts array as null.
			if(empty($parts[0]) && count($parts) > 1){
				$parts = array_slice($parts,1);
			}
			
			$this->req_controller = (isset($parts[0]) && !empty($parts[0]))? $parts[0] : $this->default_controller;
			
			//if controller part is a directory
			if(is_dir(APP_ROOT . 'controllers' . DS . $this->directory . $parts[0] ) && !empty($parts[0])){

				//Set the directory then remove element from $parts
				$this->directory = $parts[0] . DS;
				$parts = array_slice($parts, 1);
				
				//Does the requested controller exist in the sub-folder?
				if (!file_exists(APP_ROOT . 'controllers' . DS . $this->directory . @$parts[0] . '.php')){
					error('', '', '404');
				}
				
				if(count($parts) > 0){
					
					$this->req_controller = $parts[0];
					
				}else{
					$this->req_controller = $this->default_controller;
					$this->method = 'index';
				}
				
			}else{
				//make sure controller can be loaded
				if(!file_exists(APP_ROOT . 'controllers' . DS . $this->directory . $this->req_controller . '.php')){
					error('', '', '404');
				}
			}
			
			
			//if method defined
			if(isset($parts[1])){
				$this->req_method = $parts[1];
			}
			
			//No action specified, load default action
			if(empty($this->req_method)){
				$this->req_method = 'index';
			}
			

		}
		
		/*
		* Redirect to a new page
		* @string $o = the location to redirect to
		* @string $method = type of redirect (refresh|redirect)
		*/
		public function redirect($to, $method='redirect'){
			//TODO
			if(strstr($to, 'http')){
				
				if($method == 'refresh'){
					header("refresh:0; url=/$to/");
				}else{
					
					header("Location: $to");
					
				}
				
			}
			return false;
		}
		
		/**
		* Protect method from being accessed directly from URI
		*/
		function _protect_methods(){
			//Add method(s) to protected methods list, defined in routes config file.
			if($this->routes['protected_methods'] != '' && is_array($this->routes['protected_methods'])){
				foreach($this->routes['protected_methods'] as $method){
					$this->protected_methods[] = $method;
				}
				
			}else{
				error("Your protected methods config var must be array!", "Config Error");
			}
			return;
		}
		
		
	}
	
?>