<?php	
	
	if(!defined('SYS_ROOT')){exit('No direct file access allowed!');}
	
	/**
	* Xtron PHP Framework
	* A MVC style framework for building web applications
	* By: Jason Alan Delaney (http://www.xtroncms.com)
	* PHP version 5+
	*/	
	
	/**
	* Template class for using custom tags in view files
	* and the use of global themes.
	*/
	
	class Template {
		
		var $config;//Config array from app/config/
		var $tags = array();
		var $tag_files;
		var $template;//View file name
		var $theme_dir = '';//Path to theme
		var $theme_name= '';
		var $cache;
		var $tag;//Tag name for content when using theme
		var $ext = '.php';//Template file extension (not really used in current ver. 1.0.0)
		var $data;//Variables to pass through to view file
		
		function __construct($tags = array()){
			$this->config =& load_class('Config');
			//Load template config file
			$this->config->load('template');
			
			if(count($tags) > 0 && is_array($tags)){
				foreach($tags as $tag => $val){
					$this->tags[$tag] = $val;
				}
			}
			
		}
		
		/*
		* Parse method wrapper
		*/
		function view($template, $data=null, $cache=false, $tags=array()){
			$this->parse($template, $data=null, $cache=false, $tags=array());
		}
		
		/**
		* Echo view file to the browser.
		* Cache functions
		* Themes
		*/
		function parse($template, $data=null, $cache=false, $tags=array()){

			/**
			* TODO:
			* - Cache expiration
			* - Cache exists OR template exists
			* - Don't use replace_tags() if cached version is available
			*/
			
			$this->template = $template;
			$this->cache	= $cache;
			$this->data		= $data;
			
			$this->load_tags($tags);
			if($this->template_exists($template) || $cache === true){
				/**
				* Render content through theme file if using one
				*/
				if($this->theme_dir != '' && !empty($this->theme_dir)){
					//Put page contents into tag
					$this->tags[$this->tag] = $this->replace_tags();
					//Set template to theme template
					$this->template = $this->theme_dir;
					
					echo $this->replace_tags($cache);
					
				}else{
					
					echo $this->replace_tags($cache);
				
				}
				return;
			}
			
			return false;
			
		} 
		
		/**
		* Set a theme file to use. If set, all view file data is passed through the theme file.
		*/
		function use_theme($theme, $ext = '.php'){
			
			/**
			* TODO: 
			* - Theme caching
			* - Custom theme extensions '.theme', '.template','.tpl' etc
			*/
			
			$ext = (!ctype_alpha($ext))? '.php' : $ext;
			
			$theme = strtolower($theme);
			
			if(empty($theme) || $theme == ''){	
				error('Theme name can not be empty - function::use_theme()'. 'Template Error');
				return;
			}
			
			//Check if theme path is defined
			if(file_exists(APP_ROOT . 'views' . DS . basename($theme, $ext) . $ext) && strpos($theme, $ext)){
			
				$this->theme_dir = APP_ROOT . 'views' . DS . $theme;
				$theme = explode('/',$theme);
				
				$this->theme_name = end($theme);
				
			}elseif(file_exists(basename($theme, $ext) . $ext)){
			
				$this->theme_dir = str_replace($ext, '', $theme) . $ext;
				$this->theme_name = basename($theme, $ext);
				
			}else{
				
				error('Theme file "' . basename($theme, $ext) . $ext . '" cannot be found in the default directories. Try supplying full path to file.', 'Template Error');
				return;
				
			}
			
			$this->tag = str_replace(array('{','}'), '', $this->config->item('content_tag'));
			
		}
		
		
		/**
		* Load template tag files
		*/
		function load_tags($ctags=array()){
			
			if(is_array($ctags) && count($ctags) > 0){
				$this->tags = array_merge($ctags, $this->tags);
				return;
			}
			
			if(isset($this->config->config['tag_files']) && is_array($this->config->config['tag_files']) && count($this->config->config['tag_files']) > 0){
				foreach($this->config->config['tag_files'] as $file){
					if(file_exists(APP_ROOT . 'tags' . DS . 'tags.' . $file . '.php')){
						include(APP_ROOT . 'tags' . DS . 'tags.' . $file . '.php');
					}
				}
				
				if(isset($tags)){
					$this->tags = array_merge($tags, $this->tags);
					return;
				}
				//In case tag file didn't actually have a $tag array pff
				$this->tags = array();
				
			}else{
				//No tag file specified or found?
				$this->tags = array();
				return;
			}
			
		}
		
		/**
		* Checks if a given template exists
		*/
		function template_exists($tpl, $ext = '.php'){
			$tpl = preg_replace('~\.(php|tpl|template)~', '', $tpl);
			
			$ext = ($ext == '')?  $this->ext : $ext;
			
			if(file_exists(APP_ROOT . 'views' . DS . $tpl . $ext)){
				return true;
			}

			error('View file "' . $tpl . '" cannot be found.', 'Template Error: ');
			return false;
			
		}
		
		/**
		* @bool cache parsed view file? (TRUE|FALSE)
		* Replace custom tags
		*/
		private function replace_tags(){
			//Get instance of the main controller so we can load the view file to parse
			$xpf =& controller_instance();
			
			//Get raw view file from the Display Object
			$view = $xpf->display->view($this->template, $this->data, $this->cache);

			//Replace tags if caching is off
			if($this->cache == false && count($this->tags) > 0){
				//Replace data in {tag} with data from tag file
				foreach ($this->tags as $tag => $data){
					$view = preg_replace("~{+" . $tag . "+}~", $data, $view);
				}
			}
			
			return $view;
		}
		
	}
	
?>