<?php
	
	if(!defined('SYS_ROOT')){exit('No direct file access allowed!');}
	
	/**
	* PopcornPHP
	* A MVC style framework for building web applications
	* By: Jason Delaney (http://www.popcornphp.com)
	* PHP version 5+
	*
	* Validates various data 
	*/
	
	class Validate {
		
		/*
		* Global regex match binder function
		* @string $regex = he regular expression
		* @string $string = the string to match
		* @bool $count = return number of matches?
		*/
		private function _match($regex, $string, $count=false){
			$match = preg_match($regex, $string, $matches);
			
			//return number of matches
			if($count && $match) return count($matches);
			
			return $match;
		}
		
		//Checks string to be a valid email
		public function email($email){
			$regex = '/\b[A-Z0-9._%+-]+@(?:[A-Z0-9-]+\.)+[A-Z]{2,6}\b/i';
			return $this->_match($regex, $email);
		}
		
		/**
		* Validates user names. A-Z 0-9 characters. Minimum of 5 chars. Max of 20
		*/
		public function username($username){
			$regex = '/^[a-z\d_]{5,20}$/i';
			return $this->_match($regex, $username);
		}
		
		/**
		* Validates IP adresses
		*/
		public function ip($ip){
			$regex = '/^(([1-9]?[0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]).){3}([1-9]?[0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/';
			return $this->_match($regex, (string) $ip);
		}
		
		/*
		* Checks for a string to be a money value
		* @string $string = the string to check against
		* @bool $dollar = check for dollar sign? 
		*/
		public function money($string, $dollar=true){
			
			$dollar= ($dollar === true)? '?' : '';
			$regex = '/^\$' . $dollar . '(?:\d+|\d{1,3}(?:,\d{3})*)(?:\.\d{1,2}){0,1}$/';
		
			return $this->_match($regex, $string);
		}
		
		/*
		* Run custom regex. Good to use so that they use the global _match() method.
		*/
		public function regex($regex, $string){
			return $this->_match($regex, $string);
		}
		
		/**
		* Check for valid form submission against CSRF
		*/
		public function csurf_check($name='csurf'){
			if($_SERVER['REQUEST_METHOD'] == 'POST'){
				if(isset($_POST[$name])){
					if($_POST[$name] == $_SESSION[$name]){
						return;
					}
				}
				error('Access denied.', 'Validation Error: ');
			}
			return;
		}
		
		/**
		* Generate session token for form
		*/
		public function csurf_generate_token($name='csurf'){
			
			/**
			* TODO:
			* - Automatically generate HTML input into all forms.
			* - Session token timeout
			*/
			
			if(!isset($_SESSION)){
				minor_error('Sessions must be enabled in order to use the csurf functions. Set app/config/config.php "session_auto_start" to true.','ACL Error:');
			}
			
			$key = sha1(microtime() . session_id() . mt_rand());
			$_SESSION['csurf']	= $key;
			
			echo '<input type="hidden" name="' . $name . '" value="' . $key . '" />';
			
		}
		
		/**
		* Removes possible xss attack data
		* @bool notags = set to true to strip all html tags
		*/
		public function xss_destroy($data, $notags=false){
			if($notags == false){
				
				$data = htmlentities($data, ENT_QUOTES);
			
			}else{
				/**
				* Strip all HTML tags and invisible text
				* Courtesy of nadeausoftware.com
				*/
				$data = preg_replace(
				array(
				  // Remove invisible content
					'@<head[^>]*?>.*?</head>@siu',
					'@<style[^>]*?>.*?</style>@siu',
					'@<script[^>]*?.*?</script>@siu',
					'@<object[^>]*?.*?</object>@siu',
					'@<embed[^>]*?.*?</embed>@siu',
					'@<applet[^>]*?.*?</applet>@siu',
					'@<noframes[^>]*?.*?</noframes>@siu',
					'@<noscript[^>]*?.*?</noscript>@siu',
					'@<noembed[^>]*?.*?</noembed>@siu',
				  // Add line breaks before and after blocks
					'@</?((address)|(blockquote)|(center)|(del))@iu',
					'@</?((div)|(h[1-9])|(ins)|(isindex)|(p)|(pre))@iu',
					'@</?((dir)|(dl)|(dt)|(dd)|(li)|(menu)|(ol)|(ul))@iu',
					'@</?((table)|(th)|(td)|(caption))@iu',
					'@</?((form)|(button)|(fieldset)|(legend)|(input))@iu',
					'@</?((label)|(select)|(optgroup)|(option)|(textarea))@iu',
					'@</?((frameset)|(frame)|(iframe))@iu',
				),
				array(
					' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ',
					"\n\$0", "\n\$0", "\n\$0", "\n\$0", "\n\$0", "\n\$0",
					"\n\$0", "\n\$0",
				),
				$data );
				$data = strip_tags($data);
			}
			//Remove possible "javascript:" syntax from links or input
			$data = preg_replace('/javascript:/i', '', $data);
			
			return $data;
			
		}
		
		
	}
	
?>